#!/usr/bin/env perl

# Application
package GaiaLBS::BackendSelector;

use utf8;
use Mojolicious::Lite;

use feature 'say';
use Data::Dumper::Simple;
use GaiaLBS::DataStore::Redis;
use Mojo::JSON;

my $DEBUG	= 1;
$| = 1;

app->config(hypnotoad => {listen => ['http://*:3001']});


my %DS;
$DS{redis}  = GaiaLBS::DataStore::Redis->new(
	host => '127.0.0.1',
	port => 6379,
	encoding => 'utf8',
	on_error => sub { warn @_ },
);

my $json   = Mojo::JSON->new;


# Documentation browser under "/perldoc"
#~ plugin 'PODRenderer';

get '/' => sub {
	my $self = shift;
	$self->render('index');
};

get '/:foo' => sub {
	my $self = shift;
	my $foo  = $self->param('foo');
	$self->render(text => "Hello from $foo.");
};

get '/query_square/:apikey/:layer/:ident/:xstart/:ystart/:zstart/:xstop/:ystop/:zstop' => sub {
	my $self = shift;
	
	$self->render_later;
	
	my $node_info = {
		api_key	=> $self->param('apikey'),
		layer	=> $self->param('layer'),
		ident	=> $self->param('ident'),
	};
	
	say "GOT DATA in store_pos:" if $DEBUG;
	say Dumper $node_info if $DEBUG;
	
	
	
	my $cv = $DS{redis}->query_square(
		$node_info,
		$self->param('xstart'),
		$self->param('ystart'),
		$self->param('zstart'),
		$self->param('xstop'),
		$self->param('ystop'),
		$self->param('zstop'),
	);
	
	$cv->cb( sub {
		$self->render_json( \@{[ $_[0]->recv ]} );
	});
};

# input:
#	api_key => 'xxxxxxx', layer => '123qwe', ident => ‘node_ident’ 
#	loc: (x, y, z)
#	movement: ( (x1, y1, z1, x2, y2, z2), speed)
#	orientation: (360, 360)

post '/store_pos/:apikey/:layer/:ident' => sub {
	my $self = shift;
	my $pos = $self->req->param('pos');
	say Dumper $pos;
	my $data = $json->decode( $pos );
	
	$self->render_later;
	
	say Dumper( \@{[ $self->param ]} ) if $DEBUG;
	
	say "GOT DATA in store_pos:" if $DEBUG;
	say Dumper $data if $DEBUG;
	
	my $node_info = {
		api_key	=> $self->param('apikey'),
		layer	=> $self->param('layer'),
		ident	=> $self->param('ident'),
	};
	
	my $cv = $DS{redis}->store_pos(
		$node_info,
		$data->{loc},
		$data->{move},
		$data->{orien},
	);
	
	$cv->cb( sub {
		$self->render_json( \@{[ $_[0]->recv ]} );
	});
};


get '/valid/:apikey/:layer/:ident' => sub {
	my $self = shift;
	$self->render_later;
	
	my $node_info = {
		api_key	=> $self->param('apikey'),
		layer	=> $self->param('layer'),
		ident	=> $self->param('ident'),
	};
	
	my $cv = $DS{redis}->valid(
		%$node_info,
	);
	
	$cv->cb( sub {
		my ($res, $err) = $_[0]->recv;
		#~ say Dumper( $res, $err );
		$self->render_json( \@{[$res, $err]} );
	});
};


# FIXME: accept multiple target via post JSON
get '/stalk/:target_apikey/:target_layer/:target_ident/by/:apikey/:layer/:ident' => sub {
	my $self = shift;
	#~ my $data = $json->decode( $self->req->param('pos') );
	
	$self->render_later;
	
	#~ say Dumper( \@{[ $self->param ]} ) if $DEBUG;
	
	#~ say "GOT DATA in store_pos:" if $DEBUG;
	#~ say Dumper $data if $DEBUG;
	
	my $node_info = {
		api_key	=> $self->param('apikey'),
		layer	=> $self->param('layer'),
		ident	=> $self->param('ident'),
	};
	
	my $target_info = {
		api_key	=> $self->param('target_apikey'),
		layer	=> $self->param('target_layer'),
		ident	=> $self->param('target_ident'),
	};
	
	my $cv = $DS{redis}->stalk(
		$node_info,
		$target_info
	);
	
	$cv->cb( sub {
		my ($res, $err) = $_[0]->recv;
		#~ say Dumper( $res, $err );
		$self->render_json( \@{[$res, $err]} );
	});
};


get '/stalkers/:apikey/:layer/:ident' => sub {
	my $self = shift;
	
	$self->render_later;
	
	my $node_info = {
		api_key	=> $self->param('apikey'),
		layer	=> $self->param('layer'),
		ident	=> $self->param('ident'),
	};
	
	my $cv = $DS{redis}->stalkers(
		$node_info,
	);
	
	$cv->cb( sub {
		my ($res, $err) = $_[0]->recv;
		#~ say Dumper( $res, $err );
		$self->render_json( \@{[$res, $err]} );
	});
};

app->start;




__DATA__

@@ index.html.ep
% layout 'default';
% title 'Welcome';
Welcome to the Mojolicious real-time web framework!

@@ layouts/default.html.ep
<!DOCTYPE html>
<html>
  <head><title><%= title %></title></head>
  <body><%= content %></body>
</html>
